<?php
/***************************************************************************
*                            reminder.php
 *                            -------------------
 *   begin                : Wednesday, Sep 3, 2008
 *   copyright            : (C) 2001 The phpBB Group
 *   email                : support@phpbb.com
 *
 *   $Id: reminders.php
 *
 *
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/
// Written by David Colon, david@davidiq.com
// This software is designed to work with phpBB Version 2.0.23 but will work with previous versions (untested).
// Loosely based on the e-mail digests work done by Mark D. Hamill.

// Default number of days to check for inactivity is 30.
// Change to the value you'd like.
$days = 30;

define('IN_PHPBB', true);
$phpbb_root_path = './'; 

include($phpbb_root_path . 'extension.inc');
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/emailer.'.$phpEx);

$userdata = session_pagestart($user_ip, PAGE_INDEX);
init_userprefs($userdata);

//Create column if it doesn't exist
$columnexists = false;
$column_name = 'user_reminded';
$table = USERS_TABLE;

//Borrowed from phpBB3's db_tools
switch ($dbms)
{
	case 'mysql':
	case 'mysql4':
	case 'mysql_40':
	case 'mysql_41':

		$sql = "SHOW COLUMNS FROM $table";
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			// lower case just in case
			if (strtolower($row['Field']) == $column_name)
			{
				$columnexists = true;
			}
		}
	break;

	// PostgreSQL has a way of doing this in a much simpler way but would
	// not allow us to support all versions of PostgreSQL
	case 'postgres':
		$sql = "SELECT a.attname
			FROM pg_class c, pg_attribute a
			WHERE c.relname = '{$table}'
				AND a.attnum > 0
				AND a.attrelid = c.oid";
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			// lower case just in case
			if (strtolower($row['attname']) == $column_name)
			{
				$columnexists = true;
			}
		}
	break;

	// same deal with PostgreSQL, we must perform more complex operations than
	// we technically could
	case 'mssql':
		$sql = "SELECT c.name
			FROM syscolumns c
			LEFT JOIN sysobjects o ON c.id = o.id
			WHERE o.name = '{$table}'";
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			// lower case just in case
			if (strtolower($row['name']) == $column_name)
			{
				$columnexists = true;
			}
		}
	break;

	case 'oracle':
		$sql = "SELECT column_name
			FROM user_tab_columns
			WHERE table_name = '{$table}'";
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			// lower case just in case
			if (strtolower($row['column_name']) == $column_name)
			{
				$columnexists = true;
			}
		}
	break;

	case 'firebird':
		$sql = "SELECT RDB\$FIELD_NAME as FNAME
			FROM RDB\$RELATION_FIELDS
			WHERE RDB\$RELATION_NAME = '{$table}'";
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			// lower case just in case
			if (strtolower($row['fname']) == $column_name)
			{
				$columnexists = true;
			}
		}
	break;

	// ugh, SQLite
	case 'sqlite':
		$sql = "SELECT sql
			FROM sqlite_master
			WHERE type = 'table'
				AND name = '{$table}'";
		$result = $db->sql_query($sql);

		if (!$result)
		{
			$columnexists = false;
		}

		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		preg_match('#\((.*)\)#s', $row['sql'], $matches);

		$cols = trim($matches[1]);
		$col_array = preg_split('/,(?![\s\w]+\))/m', $cols);

		foreach ($col_array as $declaration)
		{
			$entities = preg_split('#\s+#', trim($declaration));
			if ($entities[0] == 'PRIMARY')
			{
				continue;
			}

			if (strtolower($entities[0]) == $column_name)
			{
				$columnexists = true;
			}
		}
	break;
}

$db->sql_freeresult($result);

if (!$columnexists)
{
    $sql = "ALTER TABLE " . USERS_TABLE . " ADD user_reminded INT(11) NULL;";
	if (!($result = $db->sql_query($sql)))
	{
		message_die(GENERAL_ERROR, 'Unable to alter the users table to add the user_reminded column.', '', __LINE__, __FILE__, $sql);
	}
	else
	{
		$message = 'user_reminded column added to the users table';	
	}
}

//Figure out which users have not visited in the past 30 days.
//If they've been reminded already then we don't send another e-mail out unless they visited since the last reminder

$currtime = time();
$previous30 = time() - ($days * 24 * 60 * 60);

$sql = "SELECT u.user_id, u.username, u.user_email, u.user_lastvisit, u.user_reminded
	FROM " . USERS_TABLE . " u 
	WHERE u.user_active = 1 AND u.user_lastvisit <= '" . $previous30 . "' AND (u.user_reminded IS NULL OR
	u.user_reminded <= u.user_lastvisit) AND u.user_email IS NOT NULL";

if ( !($result = $db->sql_query($sql)))
{
	message_die(GENERAL_ERROR, 'Unable to retrieve list of users to be reminded.', '', __LINE__, __FILE__, $sql);
}

$script_path = preg_replace('/^\/?(.*?)\/?$/', "\\1", trim($board_config['script_path']));
$server_name = trim($board_config['server_name']);
$server_protocol = ( $board_config['cookie_secure'] ) ? 'https://' : 'http://';
$server_port = ( $board_config['server_port'] <> 80 ) ? ':' . trim($board_config['server_port']) . '/' : '/';

$server_url = $server_protocol . $server_name . $server_port . $script_path;

// Figure out how many reminders have been sent
$reminders_sent = 0;

while ($row = $db->sql_fetchrow($result)) 
{
	if (!(is_object($emailer)))
	{
		$emailer = new emailer($board_config['smtp_delivery']);
	}

	$emailer->email_address(($board_config['smtp_delivery']) ? $row['user_email'] : $row['username'] . ' <' . $row['user_email'] . '>');
	$emailer->from($board_config['board_email']);
	$emailer->replyto($board_config['board_email']);
	$emailer->use_template('user_reminder', 'lang_english');
	$emailer->set_subject($row['username'] . ' we miss you at ' . $board_config['sitename'] . '!');
	$emailer->assign_vars(array(
		'SITENAME' => $board_config['sitename'],
		'USERNAME' => $row['username'],
		'U_BOARD' => $server_url,
		'U_FORGOTPW' => $server_url . '/profile.' . $phpEx . '?mode=sendpassword',
		'EMAIL_SIG' => str_replace('<br />', "\n", "-- \n" . $board_config['board_email_sig'])
		));
	
	$emailer->send();
	$emailer->reset();
	break;
	//Update last reminder date
	$sql2 = 'UPDATE ' . USERS_TABLE . ' SET user_reminded = ' . $currtime . ' WHERE user_id = ' . $row['user_id'];
	
	if (!($result2 = $db->sql_query($sql2)))
	{
		$error .= 'Unable to send reminder to: ' . $row['username'] . '</br>';
	}
	
	//Increment counter
	$reminders_sent++;
}

$db->sql_freeresult($result);

// Summary information for looking over if running this manually
echo "<html>";
echo "<head>";
echo '<title>User Reminder</title>';
echo "</head>";
echo "<body>";
echo '<h1>Results of reminders processing:</h1>';
echo "<hr />";
echo 'Total reminders sent: ' . $reminders_sent . '</br>';
echo 'Current server date/time: ' . date('m-d-Y @ h:i:s') . '</br>';
if ($message != '')
{
	echo $message . '</br>';
}
if ($error != '')
{
	echo 'Errors encountered:</br>';
	echo $error;
}
echo "</body>";
echo "</html>";

?>
